#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "../include/defs.h"
#include "../include/proto.h"
#include "../include/special-functions.h"
#include "../include/mconf.h"  

double psi2(int m, int n);

void Serial(int m, int n)
{
  int    state;
  char   assignment[7];
  double p_value1, p_value2, psim0, psim1, psim2, del1, del2;
  
  psim0 = psi2(m, n);
  psim1 = psi2(m-1, n);
  psim2 = psi2(m-2, n);
  del1 = psim0 - psim1;
  del2 = psim0 - 2.0*psim1 + psim2;
  p_value1 = igamc(pow(2,m-1)/2,del1/2.0);
  p_value2 = igamc(pow(2,m-2)/2,del2/2.0);
  /*p_value1 = gammq(pow(2,m-1)/2,del1/2.0);*/
  /*p_value2 = gammq(pow(2,m-2)/2,del2/2.0);*/
  fprintf(stats[14],"\t\t\t       SERIAL TEST\n");
  fprintf(stats[14],"\t\t---------------------------------------------\n");
  fprintf(stats[14],"\t\t COMPUTATIONAL INFORMATION:		  \n");
  fprintf(stats[14],"\t\t---------------------------------------------\n");
  fprintf(stats[14],"\t\t(a) Block length    (m) = %d\n", m);
  fprintf(stats[14],"\t\t(b) Sequence length (n) = %d\n", n);
  fprintf(stats[14],"\t\t(c) Psi_m               = %f\n", psim0);
  fprintf(stats[14],"\t\t(d) Psi_m-1             = %f\n", psim1);
  fprintf(stats[14],"\t\t(e) Psi_m-2             = %f\n", psim2);
  fprintf(stats[14],"\t\t(f) Del_1               = %f\n", del1);
  fprintf(stats[14],"\t\t(g) Del_2               = %f\n", del2);
  fprintf(stats[14],"\t\t---------------------------------------------\n");
  /*
  printf("%10.6f %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f\n",
          psim0, psim1, psim2, del1, del2, p_value1,p_value2);
  */
  if (p_value1 < ALPHA) {
     strcpy(assignment,"FAILURE");
     state = 0;
  }
  else {
     strcpy(assignment,"SUCCESS");
     state = 1;
  }
  fprintf(stats[14],"%s\t\tp_value1 = %f\n", assignment, p_value1);
  fprintf(results[14],"%f\n", p_value1);
  fprintf(grid, "%d", state);
  fflush(grid);
  if (p_value2 < ALPHA) {
     strcpy(assignment,"FAILURE");
     state = 0;
  }
  else {
     strcpy(assignment,"SUCCESS");
     state = 1;
  }
  fprintf(stats[14],"%s\t\tp_value2 = %f\n\n", assignment, p_value2);
  fprintf(results[14],"%f\n", p_value2);
  fprintf(grid, "%d", state);
  fflush(stats[14]);
  fflush(results[14]);
  fflush(grid);
  return;
}

double psi2(int m, int n)
{
  int            i, j, k, powLen;
  double         sum, numOfBlocks;
  unsigned int*  P;
  
  if ((m == 0) || (m == -1)) return 0.0;
  numOfBlocks = n;
  powLen = (int)pow(2,m+1)-1;
  if ((P = (unsigned int*)calloc(powLen,sizeof(unsigned int)))== NULL){
     fprintf(stats[14],"Serial Test:  Insufficient memory available.\n");
     fflush(stats[14]);
     return 0;
  }
  for(i = 1; i < powLen-1; i++) P[i] = 0;	  /* INITIALIZE NODES */
  for(i = 0; i < numOfBlocks; i++) {		 /* COMPUTE FREQUENCY */
     k = 1;
     /*printf("BLOCK %2d => ",i);*/
     for(j = 0; j < m; j++) {
        /*printf("%1d",epsilon[(i+j)%n].b);*/
        if (epsilon[(i+j)%n].b == 0)
           k *= 2;
        else if (epsilon[(i+j)%n].b == 1)
           k = 2*k+1;
        /*if ((i+j+1)%40 == 0)printf("\n");*/
     }
     /*printf("\n");*/
     P[k-1]++;
  }
  /* DISPLAY FREQUENCY */
  /*
  for(i = (int)pow(2,m)-1; i < (int)pow(2,m+1)-1; i++)
     printf("%5d ", P[i]);
  */
  sum = 0.0;
  for(i = (int)pow(2,m)-1; i < (int)pow(2,m+1)-1; i++)
     sum += pow(P[i],2);
  sum = (sum * pow(2,m)/(double)n) - (double)n;
  free(P);
  return sum;
}
